Package: miceDRF
Type: Package
Title: Imputation with 'mice' and Distributional Random Forests
Version: 0.1.0
Authors@R: c(person(
        "Krystyna", "Grzesiak",
        email = "krygrz11@gmail.com",
        role = c("aut", "cre"),
        comment = c(ORCID = "0000-0003-2581-7722")
    ),person(
        "Jeffrey", "Näf",
        email = "jeffrey@my-sh.ch",
        role = "aut",
        comment = c(ORCID = "0000-0003-0920-1899")))
Description: Provides a custom imputation method for the 'mice' package
    based on distributional random forests. The package implements the
    'mice.impute.DRF' method, which can be used within the standard
    'mice' workflow. Missing values are imputed by estimating
    conditional distributions with distributional random forests and
    sampling observed responses using forest weights.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/KrystynaGrzesiak/miceDRF,
        https://krystynagrzesiak.github.io/miceDRF/
BugReports: https://github.com/KrystynaGrzesiak/miceDRF/issues
Imports: drf
Suggests: mice, spelling, testthat (>= 3.0.0)
Language: en-US
Config/testthat/edition: 3
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-28 17:24:23 UTC; Krysia
Author: Krystyna Grzesiak [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2581-7722>),
  Jeffrey Näf [aut] (ORCID: <https://orcid.org/0000-0003-0920-1899>)
Maintainer: Krystyna Grzesiak <krygrz11@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-01 13:50:02 UTC
Built: R 4.5.2; ; 2026-06-01 14:44:34 UTC; unix
